#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include "CutterObject.h"

bool flag;
CutterObject *object;

GLfloat angle = -30;   /* in degrees */
GLfloat angle2 = 30;   /* in degrees */
int moving, startx, starty;

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (!flag) {
		glTranslated(-127.5,-127.5,-355);
		glTranslated(127.5,127.5,0);
		glRotated(-90, 0,0,1);
		glTranslated(-127.5,-127.5,0);

		glTranslated(127.5,0,127.5);
		glRotated(-90, 0,1,0);
		glTranslated(-127.5,0,-127.5);
		flag = true;
	}

	glPushMatrix();

	glTranslated(127.5,127.5,0);
	glRotated(angle, 0,0,1);
	glTranslated(-127.5,-127.5,0);

	glTranslated(127.5,0,127.5);
	glRotated(angle2, 0,1,0);
	glTranslated(-127.5,0,-127.5);
	
	object->Draw(true, false, true, true);

	glPopMatrix();

	glFlush();
	glutSwapBuffers();
}

void mouse(int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON) {
		if (state == GLUT_DOWN) {
			moving = 1;
			startx = x;
			starty = y;
		}
		if (state == GLUT_UP) {
			moving = 0;
		}
	}
}

void motion(int x, int y)
{
	if (moving) {
		angle = angle + (x - startx);
		angle2 = angle2 + (y - starty);
		startx = x;
		starty = y;
		glutPostRedisplay();
	}
}

void parsekey(unsigned char key, int x, int y)
{
	switch (key) {
		case 27: {
			delete object;
			exit(0);
		} break;
		case 13: break;
	}
	glutPostRedisplay();
}

void parsekey_special(int key, int x, int y)
{
	switch (key) {
		case GLUT_KEY_UP: angle2 -= 5; break;
		case GLUT_KEY_DOWN: angle2 += 5; break;
		case GLUT_KEY_RIGHT: angle += 5; break;
		case GLUT_KEY_LEFT: angle -= 5; break;
	}
	glutPostRedisplay();
}

void resize(int width, int height)
{
	int min = width < height ? width : height;

	glViewport((width-min)/2, (height-min)/2, min, min);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( /* field of view in degree */ 40.0,
    /* aspect ratio */ 1.0,
    /* Z near */ 1.0, /* Z far */ -1.0);
	//gluLookAt(0,0,5, 0,0,0, 0,1,0);
	gluLookAt(0.0, 0.0, 500.0,  /* eye is at (0,0,5) */
    0.0, 0.0, 0.0,      /* center is at (0,0,0) */
    0.0, 1.0, 0.);      /* up is in positive Y direction */

	glMatrixMode(GL_MODELVIEW);
}

int main(int argv, char **argc)
{
	if (argv != 8) {
		if (argv != 1) {
			printf("Usage: Cutter dd Va Vb Vc x0 y0 z0\n");
			printf("\n");
			printf("    Arguments:\n");
			printf("       dd - dots per cube line\n");
			printf("       Va - X component (vector N)\n");
			printf("       Vb - Y component (vector N)\n");
			printf("       Vc - Z component (vector N)\n");
			printf("       x0 - X zero point shift (vector N)\n");
			printf("       y0 - Y zero point shift (vector N)\n");
			printf("       z0 - Z zero point shift (vector N)\n");
			printf("\n");
			printf("    Ranges:\n");
			printf("       dd: 2 - ... (integer)\n");
			printf("       Va: ... - ... (integer)\n");
			printf("       Vb: ... - ... (integer)\n");
			printf("       Vc: ... - ... (integer)\n");
			printf("       x0: 0 - 255 (float)\n");
			printf("       y0: 0 - 255 (float)\n");
			printf("       z0: 0 - 255 (float)\n");
			printf("\n");
			return 1;
		} else {
			object = new CutterObject;
		}
	} else {

	if ((atoi(argc[1])) >= 2) {
		object = new CutterObject(atoi(argc[1]));
	} else {
		printf("Incorrect dd value!\n");
		return 1;
	}

	object->SetNormalVector(atoi(argc[2]), atoi(argc[3]), atoi(argc[4]));
	
	float temp;
	temp = atof(argc[5]);
	if (!((temp >= 0) && (temp <= 255))) {	
		printf("Incorrect x0 value!\n");
		return 1;
	}
	temp = atof(argc[6]);
	if (!((temp >= 0) && (temp <= 255))) {	
		printf("Incorrect y0 value!\n");
		return 1;
	}
	temp = atof(argc[7]);
	if (!((temp >= 0) && (temp <= 255))) {	
		printf("Incorrect z0 value!\n");
		return 1;
	}

	object->SetZeroPoint(atoi(argc[5]), atoi(argc[6]), atoi(argc[7]));
	}

	object->FindPlane();

	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowPosition(100, 100);
	glutInitWindowSize(400, 400);
	glutCreateWindow("Cutter");
	glutDisplayFunc(display);
	glutKeyboardFunc(parsekey);
	glutSpecialFunc(parsekey_special);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutReshapeFunc(resize);

	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);

	glutMainLoop();
	return 0;
}
