#ifndef _CUTTER_OBJECT_H
#define _CUTTER_OBJECT_H

#include <GL/glut.h>
#include <vector.h>
#include "CutterPrimitives.h"

const int kBorderLength = 256;
const int kBorderCount = 32;

class CutterObject {
public:
	CutterObject();
	CutterObject(int count);
	~CutterObject();

	void FindPlane();
	void FindSimplePlane();
	void Draw(bool doDrawPlane, bool doDrawSimplePlane
		, bool doDrawCube, bool doDrawVectors);
	void SetZeroPoint(int x, int y, int z);
	void SetNormalVector(int x, int y, int z);

private:
	int fBorderCount; //dd
	double fBorderLength; //rebro
	double fZeroPointX, fZeroPointY, fZeroPointZ; //смещение точки O
	double fNormalVectorX, fNormalVectorY, fNormalVectorZ; //вектор N
	vector<vector<vector<GLubyte> > > fCube;
	vector<vector<vector<bool> > > fCubeChecked;
	TriangleList *fPlane;

	//мини-куб
	Dot fMiniCube[8]; //точки
	//ребро
	struct Border {
		int p1,p2; //крайние точки
		bool checked; //пометка
		int link[2]; //принадлежность к граням
	} fBorders[12]; //ребра
	//грани
	int fBoundaries[6][4];

	//триангуляция
	Dot fDots[6]; //точки
	int fDotCount; //количество точек
	//новые Z-ы
	double zz0,zz1,zz2,zz3;

	double fSinA, fCosA, fSinC, fCosC; //углы A,C

	void FindDegrees(); //поиск углов поворота
	void FindZ(); //поиск новых Z для направлений i,j,k
	void FindCubes(); //поиск сечения

	double GetX(int i, int j, int k); //X
	double GetY(int i, int j, int k); //Y
	double GetZ(int i, int j, int k); //Z
	double NewZ(double x, double y, double z); //поиск новой координаты Z

	void SetRandomCube();
	void ResetCubeChecked();
	void SetMiniCubeDefaults(); //постоянные установки мини-куба
	void SetMiniCube(int i, int j, int k); //установки текущего мини-куба
	Dot FindNewDot(int rb); //поиск новой точки
	bool IsCrossed(int i); //проверка на пересечение
	int FindBorder(); //поиск пересеченного ребра
	bool IsInDots(Dot d); //проверка на принадлежность массиву dots[]
	void CiklNext(int gt, int rt); //прямой цикл
	void ShiftDots(); //сдвиг точек в массиве dots[]
	void CiklPrev(int gt, int rt); //обратный цикл
	void DoTriang(); //триангуляция
	void ResetCrossedBorders(); //сброс массива rm[]
	void StartNextLook(int ti, int tj, int tk, int num);
	void LookUpBorder(int i, int j, int k); //просмотр текущего мини-куба
	void WorkWithCube(int i, int j, int k, int parameter); //работа с текущим мини-кубом

	bool fCrossedBorders[12]; //пересеченные ребра текущего мини-куба

	void DrawCube();
	void DrawVectors();
};

#endif
